from enum import Enum, unique, auto
from typing import Optional, Union, Dict, List
from urllib.parse import quote

import qrcode
from qrcode.image.pil import PilImage
from qrcode.image.svg import SvgImage


class AuthType(Enum):
    WPA = 'WPA'
    WPA2 = 'WPA'
    WEP = 'WEP'
    nopass = 'nopass'


@unique
class ImageFormat(Enum):
    SVG = auto()
    PNG = auto()


@unique
class WifiMecardParam(Enum):
    HIDDEN = 'H'
    SSID = 'S'
    AUTH = 'T'
    PASSWORD = 'P'


@unique
class DataScheme(Enum):
    WIFI = 'WIFI'
    MAILTO = 'mailto'
    TEL = 'tel'
    SMS = 'sms'
    GEO = 'geo'


def make_mecard_data(title: str, fields: Dict[WifiMecardParam, str]) -> str:

    fields_list = list()

    for field, value in fields.items():
        fields_list.append(f'{field.value}:{value}')

    mecard_data = f'{title}:{";".join(fields_list)};;'

    return mecard_data


def make_link_data(schema: Optional[DataScheme] = None,
                   link: Optional[Union[List[str], str]] = None,
                   params: Optional[Dict[str, str]] = None) -> str:

    if isinstance(link, str):
        link = [link]

    if not link:
        link = list()

    link_str = ','.join(link)
    data = link_str

    if schema:
        data = f'{schema.value}:{data}'

    if params:
        params = {str(param): quote(str(params[param])) for param in params}
        concatenation_char = '&' if '?' in link_str else '?'
        params_list = [f'{param}={value}' for param, value in params.items()]
        params_string = '&'.join(params_list)
        data = f'{data}{concatenation_char}{params_string}'

    return data


def make_image(data: str, file_format: ImageFormat,
               **kwargs) -> Union[PilImage, SvgImage]:

    image_factory = None if file_format is ImageFormat.PNG else qrcode.image.svg.SvgImage

    qrcode_data = qrcode.QRCode(
        error_correction=qrcode.constants.ERROR_CORRECT_H)
    qrcode_data.add_data(data)
    qrcode_data.make(fit=True)

    img = qrcode_data.make_image(image_factory=image_factory, **kwargs)

    return img



from typing import Optional, Union, Dict, List
from urllib.parse import quote

from .utils import AuthType, WifiMecardParam, DataScheme, make_link_data, make_mecard_data

MECARD_SPECIAL_CHARACTERS: str = '\;,:"'


def make_wifi(ssid: Union[str, int],
              auth: AuthType = AuthType.nopass,
              password: Optional[Union[str, int]] = None,
              hidden: bool = False) -> str:

    if not isinstance(ssid, str):
        ssid = str(ssid)

    if password and not isinstance(password, str):
        password = str(password)

    for spec_char in MECARD_SPECIAL_CHARACTERS:
        ssid = ssid.replace(spec_char, f'\\{spec_char}')
        if password:
            password = password.replace(spec_char, f'\\{spec_char}')

    fields = {WifiMecardParam.AUTH: auth.value, WifiMecardParam.SSID: ssid}

    if hidden:
        fields[WifiMecardParam.HIDDEN] = 'true'

    if auth is not AuthType.nopass and password:
        fields[WifiMecardParam.PASSWORD] = password

    wifi_data = make_mecard_data(title=DataScheme.WIFI.value, fields=fields)

    return wifi_data


def make_mailto(to: str,
                subject: Optional[str] = None,
                cc: Optional[Union[List[str], str]] = None,
                bcc: Optional[Union[List[str], str]] = None,
                body: Optional[str] = None) -> str:

    data = f'{DataScheme.MAILTO.value}:{to}'
    args = list()

    if subject:
        args.append(f'subject={quote(subject)}')

    if cc:
        args.append(f'cc={quote(",".join(cc))}')

    if bcc:
        args.append(f'bcc={quote(",".join(bcc))}')

    if body:
        args.append(f'body={quote(body)}')

    if args:
        data = f'{data}?{"&".join(args)}'

    return data


def make_link(url: str, params: Optional[Dict[str, str]] = None) -> str:

    data = make_link_data(link=url, params=params)

    return data


def make_tel(tel: str) -> str:

    data = make_link_data(schema=DataScheme.TEL, link=tel)

    return data


def make_sms(recipients: Optional[List[str]] = None,
             body: Optional[str] = None) -> str:

    sms_body = dict(body=body)
    data = make_link_data(schema=DataScheme.SMS,
                          link=recipients,
                          params=sms_body)

    return data


def make_geo(latitude: float, longitude: float) -> str:

    coords = [str(latitude), str(longitude)]
    data = make_link_data(schema=DataScheme.GEO, link=coords)

    return data



from enum import Enum, unique
from os import getcwd
from os import path
from typing import Optional, Union

from fire import Fire
from qrcode.image.pil import PilImage
from qrcode.image.svg import SvgImage

from wifi_qrcode import AuthType, ImageFormat
from wifi_qrcode.makers import make_wifi, make_mailto
from wifi_qrcode.utils import make_image

DEFAULT_FILE_NAME: str = 'qrcode'
DEFAULT_FILE_FORMAT: ImageFormat = ImageFormat.PNG

QR_FILL_COLOR: str = 'black'
QR_BACK_COLOR: str = 'white'


@unique
class ValidateInputStatus(Enum):
    SUCCESS = 0
    WIFI_UNKNOWN_AUTH_TYPE = 1
    WIFI_EMPTY_PASSWORD = 2


class App:
    def __init__(self) -> None:
        self.status: ValidateInputStatus = ValidateInputStatus.SUCCESS
        self.file_format: Optional[ImageFormat] = None
        self.file_path: Optional[str] = None

    def _parse_file_name(self, file_name: str) -> str:
        try:
            file_ext = file_name.split('.')[-1]
            self.file_format = ImageFormat[file_ext.upper()]
        except KeyError:
            self.file_format = DEFAULT_FILE_FORMAT
            file_name = f'{file_name}.{DEFAULT_FILE_FORMAT.name.lower()}'

        self.file_path = path.join(getcwd(), file_name)

        print(f'Format: {self.file_format.name}')
        print(f'Path: {self.file_path}')

        return self.file_path

    def _save(self, img: Union[PilImage, SvgImage]) -> None:
        print(f'Status: {self.status.name}')

        with open(self.file_path, 'wb') as file:
            img.save(file)

        exit(self.status.value)

    def wifi(self,
             ssid: Union[str, int],
             auth: str = AuthType.WPA.name,
             password: Optional[Union[str, int]] = None,
             hidden: bool = False,
             output: str = DEFAULT_FILE_NAME) -> None:

        self._parse_file_name(output)

        auth_type = AuthType.nopass

        try:
            auth_type = AuthType[auth.upper()]

            if auth_type is not AuthType.nopass and not password:
                self.status = ValidateInputStatus.WIFI_EMPTY_PASSWORD
        except KeyError:
            self.status = ValidateInputStatus.WIFI_UNKNOWN_AUTH_TYPE

        if self.status is ValidateInputStatus.SUCCESS:
            data = make_wifi(ssid=ssid,
                             auth=auth_type,
                             password=password,
                             hidden=hidden)

            print(f'Data: {data}')

            img = make_image(data=data,
                             file_format=self.file_format,
                             fill_color=QR_FILL_COLOR,
                             back_color=QR_BACK_COLOR)

            self._save(img)

    def mailto(self,
               to: str,
               subject: Optional[str] = None,
               cc: Optional[str] = None,
               bcc: Optional[str] = None,
               body: Optional[str] = None,
               output: str = DEFAULT_FILE_NAME) -> None:

        self._parse_file_name(output)

        if cc:
            cc = cc.split(',')

        if bcc:
            bcc = bcc.split(',')

        data = make_mailto(to=to, subject=subject, cc=cc, bcc=bcc, body=body)

        print(f'Data: {data}')

        img = make_image(data=data,
                         file_format=self.file_format,
                         fill_color=QR_FILL_COLOR,
                         back_color=QR_BACK_COLOR)

        self._save(img)


def main():
    app = App()
    Fire(app)


if __name__ == '__main__':
    main()
